@@ -116,7 +116,7 @@ pub mod dp {
         }
     }
 
-    pub fn filter_j_idx(n: usize, a:  &Vec<u32>) -> (Vec<usize>, u32){
+    fn filter_j_idx(n: usize, a:  &Vec<u32>) -> (Vec<usize>, u32){
         // a_min is the minimum number in an except for zero.
         let mut a_min = a.iter().max().unwrap();
         let mut a_no_zero: Vec<u32> = vec![];
@@ -218,6 +218,120 @@ pub mod dp {
         rec(&dp, &a, a_length, n, &mut route, &mut answer, &a_min);
         answer
     }
+
+    fn vec_remove(a: &mut Vec<i32>, v: i32){
+        let index = a.iter().position(|x| *x == v).unwrap();
+        a.remove(index);
+    }
+
+    /// Finds the intergers from two vectors that sum to the same value.
+    /// # Example
+    /// ```
+    /// use std::collections::VecDeque;
+    /// use subset_sum::dp::sequence_matcher;
+    /// let answer = sequence_matcher(&mut vec![3, 5, 7], &mut vec![1, 5, -3, 4, 5, 3]);
+    /// assert_eq!(answer, vec![
+    ///     vec![
+    ///         (VecDeque::from(vec![3]), 3),
+    ///         (VecDeque::from(vec![5]), 5), 
+    ///         (VecDeque::from(vec![1, -3, 4, 5]), 7), 
+    ///      ],
+    ///     vec![
+    ///         (VecDeque::from(vec![3]), 3),
+    ///         (VecDeque::from(vec![1, 4]), 5), 
+    ///         (VecDeque::from(vec![-3, 5, 5]), 7), 
+    ///      ],
+    ///     vec![
+    ///         (VecDeque::from(vec![1, -3, 5]), 3),
+    ///         (VecDeque::from(vec![5]), 5), 
+    ///         (VecDeque::from(vec![4, 3]), 7), 
+    ///      ],
+    /// ]);
+    /// 
+    /// let answer_unchanged: Vec<Vec<(VecDeque<i32>, i32)>> = Vec::new();
+    /// let answer = sequence_matcher(&mut vec![10, 20], &mut vec![9, 21]);
+    /// assert_eq!(answer, answer_unchanged);
+    /// ```
+    pub fn sequence_matcher(key: &mut Vec<i32>, targets: &mut Vec<i32>) -> Vec<Vec<(VecDeque<i32>, i32)>>{
+        let mut group: Vec<(VecDeque<i32>, i32)> = Vec::new();
+        let mut answer: Vec<Vec<(VecDeque<i32>, i32)>> = Vec::new();
+        sequence_matcher_core(key, targets, &mut group, &mut answer);
+        answer
+    }
+
+    fn sequence_matcher_core(key: &mut Vec<i32>, targets: &mut Vec<i32>, 
+            group: &mut Vec<(VecDeque<i32>, i32)>,
+            answer: &mut Vec<Vec<(VecDeque<i32>, i32)>>
+        ){
+        if key.len() == 0 && targets.len() == 0 {
+            answer.push(group.clone());
+            return;
+        } 
+        if key.len() == 0 && targets.len() > 0 {
+            return;
+        }
+        if key.len() > 0 && targets.len() == 0 {
+            return;
+        }
+
+        let set_: Vec<VecDeque<i32>> = find_subset(&targets, key[0]);
+        for set in set_ {
+            group.push((set.clone(), key[0]));
+            let i2 = key[0].clone();
+            for el in set.clone(){
+                vec_remove(targets, el);
+            }
+            vec_remove(key, key[0]);
+
+            sequence_matcher_core(key, targets, group, answer);
+            group.pop();
+            for el in set.clone(){
+                targets.push(el);
+            }
+            key.push(i2);
+        }
+    }
+
+    #[test]
+    fn test_sequence_matcher(){
+
+        let answer = sequence_matcher(&mut vec![1, 6], &mut vec![1, 3, 3]);
+        assert_eq!(answer, vec![vec![(VecDeque::from(vec![1]), 1), (VecDeque::from(vec![3, 3]), 6)]]);
+
+        let answer = sequence_matcher(&mut vec![10, 20], &mut vec![-10, 20, 16, 4]);
+        // let answer = transpose(answer);
+        assert_eq!(answer, vec![
+            vec![
+                (VecDeque::from(vec![-10, 20]), 10),
+                (VecDeque::from(vec![16, 4]), 20), 
+            ],
+        ]);
+
+        let answer_unchanged: Vec<Vec<(VecDeque<i32>, i32)>> = Vec::new();
+
+        let answer = sequence_matcher(&mut vec![10, 20], &mut vec![10, 21]);
+        assert_eq!(answer, answer_unchanged);
+
+        let answer = sequence_matcher(&mut vec![3, 5, 7], &mut vec![1, 5, -3, 4, 5, 3]);
+        assert_eq!(answer, vec![
+            vec![
+                (VecDeque::from(vec![3]), 3),
+                (VecDeque::from(vec![5]), 5), 
+                (VecDeque::from(vec![1, -3, 4, 5]), 7), 
+             ],
+            vec![
+                (VecDeque::from(vec![3]), 3),
+                (VecDeque::from(vec![1, 4]), 5), 
+                (VecDeque::from(vec![-3, 5, 5]), 7), 
+             ],
+            vec![
+                (VecDeque::from(vec![1, -3, 5]), 3),
+                (VecDeque::from(vec![5]), 5), 
+                (VecDeque::from(vec![4, 3]), 7), 
+             ],
+        ]);
+
+    }
 }
 
 
